package za.org.treehouse.hypoalarm;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.PowerManager;
import android.preference.PreferenceManager;
import android.util.Log;

import java.util.Calendar;

// TODO sound audible alarm -- see AlarmKlaxon.java
// TODO Lower alarm volume if in phone call (and detect phone call!)
// TODO set another alarm for the next half-hour (or grace_period / 2)?

public class AlarmReceiver extends BroadcastReceiver {
    SharedPreferences sharedPref;
    private AlarmManager alarmManager, graceManager;
    private PendingIntent alarmPendingIntent, gracePendingIntent;

    @Override
    public void onReceive(Context context, Intent intent) {
        sharedPref = PreferenceManager.getDefaultSharedPreferences(context);
        Boolean alarmActive = sharedPref.getBoolean(context.getString(R.string.AlarmActivePref), true);

        if (alarmActive) {
            PowerManager pm = (PowerManager) context.getSystemService(Context.POWER_SERVICE);
            PowerManager.WakeLock partialWl = pm.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, "AlarmReceiver");
            partialWl.acquire();

            // Disable any current notifications
            //context.stopService(new Intent(context, AlarmNotify.class));

            // Set a grace period alarm to send SMS
            int gracePeriod = sharedPref.getInt(context.getString(R.string.GracePeriodPref), 60);
            // TODO remove this
            gracePeriod = 1;

            Calendar graceCal = Calendar.getInstance();
            graceCal.set(Calendar.SECOND, 0);
            graceCal.add(Calendar.MINUTE, gracePeriod);
            graceManager = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
            Intent graceIntent = new Intent(context, GraceReceiver.class);
            gracePendingIntent = PendingIntent.getBroadcast(context, MainActivity.GRACE_REQUEST, graceIntent, 0);
            graceManager.cancel(gracePendingIntent);
            graceManager.setExact(AlarmManager.RTC_WAKEUP, graceCal.getTimeInMillis(), gracePendingIntent);
            Log.d("AlarmReceiver", "Setting grace alarm for " + MainActivity.debugDate(graceCal));

            // Allow user to acknowledge alarm and cancel grace alarm
            Intent alertActivityIntent = new Intent(context, AlarmAlertActivity.class);
            alertActivityIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK |
                    Intent.FLAG_ACTIVITY_EXCLUDE_FROM_RECENTS);
            context.startActivity(alertActivityIntent);


            // Reset for tomorrow; as of API 19, setRepeating() is inexact, so we use setExact()
            String alarmTimeStr = sharedPref.getString(context.getString(R.string.AlarmTimePref), null);
            // Calendar automatically advances the day since alarmTimeStr is now in the past.
            Calendar cal = MainActivity.TimeStringToCalendar(alarmTimeStr);
            alarmManager = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
            alarmPendingIntent = PendingIntent.getBroadcast(context, MainActivity.ALARM_REQUEST, intent, 0);
            alarmManager.cancel(alarmPendingIntent);
            // TODO use set() for older APIs
            alarmManager.setExact(AlarmManager.RTC_WAKEUP, cal.getTimeInMillis(), alarmPendingIntent);
            Log.d("AlarmReceiver", "Resetting alarm for " + MainActivity.debugDate(cal));

            partialWl.release();
        }
    }
}
